@@ -51,9 +51,6 @@
 
 #include "Locker.h"
 
-
-#include "common/config.h"
-
 #define dout_subsys ceph_subsys_mds
 #undef DOUT_COND
 #define DOUT_COND(cct, l) (l<=cct->_conf->debug_mds || l <= cct->_conf->debug_mds_log \
@@ -404,9 +401,6 @@ void EMetaBlob::fullbit::update_inode(MDS *mds, CInode *in)
       in->force_dirfrags();
     }
 
-    delete in->default_layout;
-    in->default_layout = dir_layout;
-    dir_layout = NULL;
     /*
      * we can do this before linking hte inode bc the split_at would
      * be a no-op.. we have no children (namely open snaprealms) to
@@ -831,12 +825,17 @@ void ESession::replay(MDS *mds)
       dout(10) << " opened session " << session->inst << dendl;
     } else {
       session = mds->sessionmap.get_session(client_inst.name);
-      if (session->connection == NULL) {
-	dout(10) << " removed session " << session->inst << dendl;
-	mds->sessionmap.remove_session(session);
+      if (session) { // there always should be a session, but there's a bug
+	if (session->connection == NULL) {
+	  dout(10) << " removed session " << session->inst << dendl;
+	  mds->sessionmap.remove_session(session);
+	} else {
+	  session->clear();    // the client has reconnected; keep the Session, but reset
+	  dout(10) << " reset session " << session->inst << " (they reconnected)" << dendl;
+	}
       } else {
-	session->clear();    // the client has reconnected; keep the Session, but reset
-	dout(10) << " reset session " << session->inst << " (they reconnected)" << dendl;
+	mds->clog.error() << "replayed stray Session close event for " << client_inst
+			  << " from time " << stamp << ", ignoring";
       }
     }
   }
@@ -1209,6 +1208,7 @@ void ESubtreeMap::replay(MDS *mds)
        p != subtrees.end();
        ++p) {
     CDir *dir = mds->mdcache->get_dirfrag(p->first);
+    assert(dir);
     if (ambiguous_subtrees.count(p->first)) {
       // ambiguous!
       mds->mdcache->add_ambiguous_import(p->first, p->second);
@@ -1244,6 +1244,7 @@ void EFragment::replay(MDS *mds)
   switch (op) {
   case OP_PREPARE:
     mds->mdcache->uncommitted_fragments.insert(desc);
+    // fall-thru
   case OP_ONESHOT:
     if (in)
       mds->mdcache->adjust_dir_fragments(in, basefrag, bits, resultfrags, waiters, true);
@@ -1322,6 +1323,7 @@ void EImportStart::replay(MDS *mds)
 
   // set auth partially to us so we don't trim it
   CDir *dir = mds->mdcache->get_dirfrag(base);
+  assert(dir);
   mds->mdcache->adjust_bounded_subtree_auth(dir, bounds, pair<int,int>(mds->get_nodeid(), mds->get_nodeid()));
 
   // open client sessions?
@@ -1348,10 +1350,11 @@ void EImportFinish::replay(MDS *mds)
 {
   if (mds->mdcache->have_ambiguous_import(base)) {
     dout(10) << "EImportFinish.replay " << base << " success=" << success << dendl;
-    if (success) 
+    if (success) {
       mds->mdcache->finish_ambiguous_import(base);
-    else {
+    } else {
       CDir *dir = mds->mdcache->get_dirfrag(base);
+      assert(dir);
       vector<dirfrag_t> bounds;
       mds->mdcache->get_ambiguous_import_bounds(base, bounds);
       mds->mdcache->adjust_bounded_subtree_auth(dir, bounds, CDIR_AUTH_UNDEF);
