@@ -18,6 +18,10 @@
 #include <stdlib.h>
 #include <ostream>
 
+#include <boost/intrusive_ptr.hpp>
+// Because intusive_ptr clobbers our assert...
+#include "include/assert.h"
+
 #include "include/types.h"
 #include "include/buffer.h"
 #include "common/Throttle.h"
@@ -261,6 +265,7 @@ public:
     rx_buffers.erase(tid);
   }
 };
+typedef boost::intrusive_ptr<Connection> ConnectionRef;
 
 
 
@@ -417,6 +422,10 @@ public:
     footer.data_crc = data.crc32c(0);
   }
 
+  virtual int get_cost() const {
+    return data.length();
+  }
+
   // type
   int get_type() const { return header.type; }
   void set_type(int t) { header.type = t; }
@@ -466,6 +475,7 @@ public:
 
   void encode(uint64_t features, bool datacrc);
 };
+typedef boost::intrusive_ptr<Message> MessageRef;
 
 extern Message *decode_message(CephContext *cct, ceph_msg_header &header,
 			       ceph_msg_footer& footer, bufferlist& front,
