@@ -14,6 +14,7 @@
 
 #include "Objecter.h"
 #include "osd/OSDMap.h"
+#include "Filer.h"
 
 #include "mon/MonClient.h"
 
@@ -120,9 +121,8 @@ enum {
 
 // messages ------------------------------
 
-void Objecter::init()
+void Objecter::init_unlocked()
 {
-  assert(client_lock.is_locked());
   assert(!initialized);
 
   if (!logger) {
@@ -208,6 +208,12 @@ void Objecter::init()
     lderr(cct) << "error registering admin socket command: "
 	       << cpp_strerror(-ret) << dendl;
   }
+}
+
+void Objecter::init_locked()
+{
+  assert(client_lock.is_locked());
+  assert(!initialized);
 
   schedule_tick();
   maybe_request_map();
@@ -215,7 +221,7 @@ void Objecter::init()
   initialized = true;
 }
 
-void Objecter::shutdown() 
+void Objecter::shutdown_locked() 
 {
   assert(client_lock.is_locked());
   assert(initialized);
@@ -231,7 +237,10 @@ void Objecter::shutdown()
     timer.cancel_event(tick_event);
     tick_event = NULL;
   }
+}
 
+void Objecter::shutdown_unlocked()
+{
   if (m_request_state_hook) {
     AdminSocket* admin_socket = cct->get_admin_socket();
     admin_socket->unregister_command("objecter_requests");
@@ -256,6 +265,8 @@ void Objecter::send_linger(LingerOp *info)
 		 onack, oncommit,
 		 info->pobjver);
   o->snapid = info->snap;
+  o->snapc = info->snapc;
+  o->mtime = info->mtime;
 
   // do not resend this; we will send a new op to reregister
   o->should_resend = false;
@@ -263,7 +274,7 @@ void Objecter::send_linger(LingerOp *info)
   if (info->session) {
     int r = recalc_op_target(o);
     if (r == RECALC_OP_TARGET_POOL_DNE) {
-      linger_check_for_latest_map(info);
+      _send_linger_map_check(info);
     }
   }
 
@@ -286,7 +297,6 @@ void Objecter::send_linger(LingerOp *info)
     if (info->session)
       s->linger_ops.push_back(&info->session_item);
   }
-  info->registering = true;
 
   logger->inc(l_osdc_linger_send);
 }
@@ -312,7 +322,6 @@ void Objecter::_linger_commit(LingerOp *info, int r)
 
   // only tell the user the first time we do this
   info->registered = true;
-  info->registering = false;
   info->pobjver = NULL;
 }
 
@@ -328,11 +337,43 @@ void Objecter::unregister_linger(uint64_t linger_id)
   }
 }
 
-tid_t Objecter::linger(const object_t& oid, const object_locator_t& oloc, 
-		       ObjectOperation& op,
-		       snapid_t snap, bufferlist& inbl, bufferlist *poutbl, int flags,
-		       Context *onack, Context *onfinish,
-		       eversion_t *objver)
+tid_t Objecter::linger_mutate(const object_t& oid, const object_locator_t& oloc,
+			      ObjectOperation& op,
+			      const SnapContext& snapc, utime_t mtime,
+			      bufferlist& inbl, int flags,
+			      Context *onack, Context *oncommit,
+			      eversion_t *objver)
+{
+  LingerOp *info = new LingerOp;
+  info->oid = oid;
+  info->oloc = oloc;
+  if (info->oloc.key == oid)
+    info->oloc.key.clear();
+  info->snapc = snapc;
+  info->mtime = mtime;
+  info->flags = flags | CEPH_OSD_FLAG_WRITE;
+  info->ops = op.ops;
+  info->inbl = inbl;
+  info->poutbl = NULL;
+  info->pobjver = objver;
+  info->on_reg_ack = onack;
+  info->on_reg_commit = oncommit;
+
+  info->linger_id = ++max_linger_id;
+  linger_ops[info->linger_id] = info;
+
+  logger->set(l_osdc_linger_active, linger_ops.size());
+
+  send_linger(info);
+
+  return info->linger_id;
+}
+
+tid_t Objecter::linger_read(const object_t& oid, const object_locator_t& oloc,
+			    ObjectOperation& op,
+			    snapid_t snap, bufferlist& inbl, bufferlist *poutbl, int flags,
+			    Context *onfinish,
+			    eversion_t *objver)
 {
   LingerOp *info = new LingerOp;
   info->oid = oid;
@@ -345,7 +386,6 @@ tid_t Objecter::linger(const object_t& oid, const object_locator_t& oloc,
   info->inbl = inbl;
   info->poutbl = poutbl;
   info->pobjver = objver;
-  info->on_reg_ack = onack;
   info->on_reg_commit = onfinish;
 
   info->linger_id = ++max_linger_id;
@@ -392,10 +432,10 @@ void Objecter::scan_requests(bool skipped_map,
 			     list<LingerOp*>& need_resend_linger)
 {
   // check for changed linger mappings (_before_ regular ops)
-  for (map<tid_t,LingerOp*>::iterator p = linger_ops.begin();
-       p != linger_ops.end();
-       p++) {
-    LingerOp *op = p->second;
+  map<tid_t,LingerOp*>::iterator lp = linger_ops.begin();
+  while (lp != linger_ops.end()) {
+    LingerOp *op = lp->second;
+    ++lp;   // check_linger_pool_dne() may touch linger_ops; prevent iterator invalidation
     ldout(cct, 10) << " checking linger op " << op->linger_id << dendl;
     int r = recalc_linger_op_target(op);
     switch (r) {
@@ -409,16 +449,16 @@ void Objecter::scan_requests(bool skipped_map,
       linger_cancel_map_check(op);
       break;
     case RECALC_OP_TARGET_POOL_DNE:
-      linger_check_for_latest_map(op);
+      check_linger_pool_dne(op);
       break;
     }
   }
 
   // check for changed request mappings
-  for (hash_map<tid_t,Op*>::iterator p = ops.begin();
-       p != ops.end();
-       ++p) {
+  map<tid_t,Op*>::iterator p = ops.begin();
+  while (p != ops.end()) {
     Op *op = p->second;
+    ++p;   // check_op_pool_dne() may touch ops; prevent iterator invalidation
     ldout(cct, 10) << " checking op " << op->tid << dendl;
     int r = recalc_op_target(op);
     switch (r) {
@@ -432,7 +472,7 @@ void Objecter::scan_requests(bool skipped_map,
       op_cancel_map_check(op);
       break;
     case RECALC_OP_TARGET_POOL_DNE:
-      op_check_for_latest_map(op);
+      check_op_pool_dne(op);
       break;
     }
   }
@@ -544,7 +584,7 @@ void Objecter::handle_osd_map(MOSDMap *m)
   // unpause requests?
   if ((was_pauserd && !pauserd) ||
       (was_pausewr && !pausewr))
-    for (hash_map<tid_t,Op*>::iterator p = ops.begin();
+    for (map<tid_t,Op*>::iterator p = ops.begin();
 	 p != ops.end();
 	 p++) {
       Op *op = p->second;
@@ -593,10 +633,15 @@ void Objecter::handle_osd_map(MOSDMap *m)
   m->put();
 
   monc->sub_got("osdmap", osdmap->get_epoch());
+
+  if (!waiting_for_map.empty())
+    maybe_request_map();
 }
 
 void Objecter::C_Op_Map_Latest::finish(int r)
 {
+  lgeneric_subdout(objecter->cct, objecter, 10) << "op_map_latest r=" << r << " tid=" << tid
+						<< " latest " << latest << dendl;
   if (r == -EAGAIN) {
     // ignore callback; we will retry in resend_mon_ops()
     return;
@@ -607,22 +652,55 @@ void Objecter::C_Op_Map_Latest::finish(int r)
   map<tid_t, Op*>::iterator iter =
     objecter->check_latest_map_ops.find(tid);
   if (iter == objecter->check_latest_map_ops.end()) {
+    lgeneric_subdout(objecter->cct, objecter, 10) << "op_map_latest op " << tid << " not found" << dendl;
     return;
   }
 
   Op *op = iter->second;
   objecter->check_latest_map_ops.erase(iter);
 
-  if (r == 0) { // we had the latest map
-    if (op->onack) {
-      op->onack->complete(-ENOENT);
+  lgeneric_subdout(objecter->cct, objecter, 20) << "op_map_latest op " << op << dendl;
+
+  if (op->map_dne_bound == 0)
+    op->map_dne_bound = latest;
+
+  objecter->check_op_pool_dne(op);
+}
+
+void Objecter::check_op_pool_dne(Op *op)
+{
+  ldout(cct, 10) << "check_op_pool_dne tid " << op->tid
+		 << " current " << osdmap->get_epoch()
+		 << " map_dne_bound " << op->map_dne_bound
+		 << dendl;
+  if (op->map_dne_bound > 0) {
+    if (osdmap->get_epoch() >= op->map_dne_bound) {
+      // we had a new enough map
+      ldout(cct, 10) << "check_op_pool_dne tid " << op->tid
+		     << " concluding pool " << op->pgid.pool() << " dne"
+		     << dendl;
+      if (op->onack) {
+	op->onack->complete(-ENOENT);
+      }
+      if (op->oncommit) {
+	op->oncommit->complete(-ENOENT);
+      }
+      op->session_item.remove_myself();
+      ops.erase(op->tid);
+      delete op;
     }
-    if (op->oncommit) {
-      op->oncommit->complete(-ENOENT);
-    }
-    op->session_item.remove_myself();
-    objecter->ops.erase(op->tid);
-    delete op;
+  } else {
+    _send_op_map_check(op);
+  }
+}
+
+void Objecter::_send_op_map_check(Op *op)
+{
+  // ask the monitor
+  if (check_latest_map_ops.count(op->tid) == 0) {
+    check_latest_map_ops[op->tid] = op;
+    C_Op_Map_Latest *c = new C_Op_Map_Latest(this, op->tid);
+    monc->get_version("osdmap", &c->latest, NULL, c);
   }
 }
 
@@ -643,32 +721,44 @@ void Objecter::C_Linger_Map_Latest::finish(int r)
 
   LingerOp *op = iter->second;
   objecter->check_latest_map_lingers.erase(iter);
-
-  if (r == 0) { // we had the latest map
-    if (op->on_reg_ack) {
-      op->on_reg_ack->complete(-ENOENT);
-    }
-    if (op->on_reg_commit) {
-      op->on_reg_commit->complete(-ENOENT);
-    }
-    objecter->unregister_linger(op->linger_id);
-  }
   op->put();
+
+  if (op->map_dne_bound == 0)
+    op->map_dne_bound = latest;
+
+  objecter->check_linger_pool_dne(op);
 }
 
-void Objecter::op_check_for_latest_map(Op *op)
+void Objecter::check_linger_pool_dne(LingerOp *op)
 {
-  check_latest_map_ops[op->tid] = op;
-  monc->is_latest_map("osdmap", osdmap->get_epoch(),
-		      new C_Op_Map_Latest(this, op->tid));
+  ldout(cct, 10) << "check_linger_pool_dne linger_id " << op->linger_id
+		 << " current " << osdmap->get_epoch()
+		 << " map_dne_bound " << op->map_dne_bound
+		 << dendl;
+  if (op->map_dne_bound > 0) {
+    if (osdmap->get_epoch() >= op->map_dne_bound) {
+      if (op->on_reg_ack) {
+	op->on_reg_ack->complete(-ENOENT);
+      }
+      if (op->on_reg_commit) {
+	op->on_reg_commit->complete(-ENOENT);
+      }
+      unregister_linger(op->linger_id);
+    }
+  } else {
+    _send_linger_map_check(op);
+  }
 }
 
-void Objecter::linger_check_for_latest_map(LingerOp *op)
+void Objecter::_send_linger_map_check(LingerOp *op)
 {
-  op->get();
-  check_latest_map_lingers[op->linger_id] = op;
-  monc->is_latest_map("osdmap", osdmap->get_epoch(),
-		      new C_Linger_Map_Latest(this, op->linger_id));
+  // ask the monitor
+  if (check_latest_map_lingers.count(op->linger_id) == 0) {
+    op->get();
+    check_latest_map_lingers[op->linger_id] = op;
+    C_Linger_Map_Latest *c = new C_Linger_Map_Latest(this, op->linger_id);
+    monc->get_version("osdmap", &c->latest, NULL, c);
+  }
 }
 
 void Objecter::op_cancel_map_check(Op *op)
@@ -749,7 +839,7 @@ void Objecter::wait_for_osd_map()
 }
 
 
-void Objecter::maybe_request_map(epoch_t epoch)
+void Objecter::maybe_request_map()
 {
   int flag = 0;
   if (osdmap->test_flag(CEPH_OSDMAP_FULL)) {
@@ -758,13 +848,16 @@ void Objecter::maybe_request_map(epoch_t epoch)
     ldout(cct, 10) << "maybe_request_map subscribing (onetime) to next osd map" << dendl;
     flag = CEPH_SUBSCRIBE_ONETIME;
   }
-  if (!epoch) {
-    epoch = osdmap->get_epoch() ? osdmap->get_epoch()+1 : 0;
-  }
+  epoch_t epoch = osdmap->get_epoch() ? osdmap->get_epoch()+1 : 0;
   if (monc->sub_want("osdmap", epoch, flag))
     monc->renew_subs();
 }
 
+void Objecter::wait_for_new_map(Context *c, epoch_t epoch, int err)
+{
+  waiting_for_map[epoch].push_back(pair<Context *, int>(c, err));
+  maybe_request_map();
+}
 
 void Objecter::kick_requests(OSDSession *session)
 {
@@ -823,7 +916,7 @@ void Objecter::tick()
   cutoff -= cct->_conf->objecter_timeout;  // timeout
 
   unsigned laggy_ops = 0;
-  for (hash_map<tid_t,Op*>::iterator p = ops.begin();
+  for (map<tid_t,Op*>::iterator p = ops.begin();
        p != ops.end();
        p++) {
     Op *op = p->second;
@@ -886,15 +979,15 @@ void Objecter::resend_mon_ops()
   for (map<tid_t, Op*>::iterator p = check_latest_map_ops.begin();
        p != check_latest_map_ops.end();
        ++p) {
-    monc->is_latest_map("osdmap", osdmap->get_epoch(),
-			new C_Op_Map_Latest(this, p->second->tid));
+    C_Op_Map_Latest *c = new C_Op_Map_Latest(this, p->second->tid);
+    monc->get_version("osdmap", &c->latest, NULL, c);
   }
 
   for (map<uint64_t, LingerOp*>::iterator p = check_latest_map_lingers.begin();
        p != check_latest_map_lingers.end();
        ++p) {
-    monc->is_latest_map("osdmap", osdmap->get_epoch(),
-			new C_Linger_Map_Latest(this, p->second->linger_id));
+    C_Linger_Map_Latest *c = new C_Linger_Map_Latest(this, p->second->linger_id);
+    monc->get_version("osdmap", &c->latest, NULL, c);
   }
 }
 
@@ -1020,7 +1113,7 @@ tid_t Objecter::_op_submit(Op *op)
   }
 
   if (check_for_latest_map) {
-    op_check_for_latest_map(op);
+    _send_op_map_check(op);
   }
 
   ldout(cct, 5) << num_unacked << " unacked, " << num_uncommitted << " uncommitted" << dendl;
@@ -1045,7 +1138,11 @@ int Objecter::recalc_op_target(Op *op)
 {
   vector<int> acting;
   pg_t pgid = op->pgid;
-  if (!op->precalc_pgid) {
+  if (op->precalc_pgid) {
+    ldout(cct, 10) << "recalc_op_target have " << pgid << " pool " << osdmap->have_pg_pool(pgid.pool()) << dendl;
+    if (!osdmap->have_pg_pool(pgid.pool()))
+      return RECALC_OP_TARGET_POOL_DNE;
+  } else {
     int ret = osdmap->object_locator_to_pg(op->oid, op->oloc, pgid);
     if (ret == -ENOENT)
       return RECALC_OP_TARGET_POOL_DNE;
@@ -1206,6 +1303,8 @@ void Objecter::send_op(Op *op)
 
   if (op->priority)
     m->set_priority(op->priority);
+  else
+    m->set_priority(cct->_conf->osd_client_op_priority);
 
   logger->inc(l_osdc_op_send);
   logger->inc(l_osdc_op_send_bytes, m->get_data().length());
@@ -1221,7 +1320,7 @@ int Objecter::calc_op_budget(Op *op)
        ++i) {
     if (i->op.op & CEPH_OSD_OP_MODE_WR) {
       op_budget += i->indata.length();
-    } else if (i->op.op & CEPH_OSD_OP_MODE_RD) {
+    } else if (ceph_osd_op_mode_read(i->op.op)) {
       if (ceph_osd_op_type_data(i->op.op)) {
         if ((int64_t)i->op.extent.length > 0)
 	  op_budget += (int64_t)i->op.extent.length;
@@ -1337,6 +1436,7 @@ void Objecter::handle_osd_op_reply(MOSDOpReply *m)
     if (*ph) {
       ldout(cct, 10) << " op " << i << " handler " << *ph << dendl;
       (*ph)->complete(p->rval);
+      *ph = NULL;
     }
   }
 
@@ -1859,107 +1959,27 @@ void Objecter::_sg_read_finish(vector<ObjectExtent>& extents, vector<bufferlist>
 			       bufferlist *bl, Context *onfinish)
 {
   // all done
-  uint64_t bytes_read = 0;
-  
   ldout(cct, 15) << "_sg_read_finish" << dendl;
 
   if (extents.size() > 1) {
-    /** FIXME This doesn't handle holes efficiently.
-     * It allocates zero buffers to fill whole buffer, and
-     * then discards trailing ones at the end.
-     *
-     * Actually, this whole thing is pretty messy with temporary bufferlist*'s all over
-     * the heap. 
-     */
-    
-    // map extents back into buffer
-    map<uint64_t, bufferlist*> by_off;  // buffer offset -> bufferlist
-    
-    // for each object extent...
+    Striper::StripedReadResult r;
     vector<bufferlist>::iterator bit = resultbl.begin();
     for (vector<ObjectExtent>::iterator eit = extents.begin();
 	 eit != extents.end();
 	 eit++, bit++) {
-      bufferlist& ox_buf = *bit;
-      unsigned ox_len = ox_buf.length();
-      unsigned ox_off = 0;
-      assert(ox_len <= eit->length);           
-      
-      // for each buffer extent we're mapping into...
-      for (map<uint64_t, uint64_t>::iterator bit = eit->buffer_extents.begin();
-	   bit != eit->buffer_extents.end();
-	   bit++) {
-	ldout(cct, 21) << " object " << eit->oid
-		 << " extent " << eit->offset << "~" << eit->length
-		 << " : ox offset " << ox_off
-		 << " -> buffer extent " << bit->first << "~" << bit->second << dendl;
-	by_off[bit->first] = new bufferlist;
-	
-	if (ox_off + bit->second <= ox_len) {
-	  // we got the whole bx
-	  by_off[bit->first]->substr_of(ox_buf, ox_off, bit->second);
-	  if (bytes_read < bit->first + bit->second) 
-	    bytes_read = bit->first + bit->second;
-	} else if (ox_off + bit->second > ox_len && ox_off < ox_len) {
-	  // we got part of this bx
-	  by_off[bit->first]->substr_of(ox_buf, ox_off, (ox_len-ox_off));
-	  if (bytes_read < bit->first + ox_len-ox_off) 
-	    bytes_read = bit->first + ox_len-ox_off;
-	  
-	  // zero end of bx
-	  ldout(cct, 21) << "  adding some zeros to the end " << ox_off + bit->second-ox_len << dendl;
-	  bufferptr z(ox_off + bit->second - ox_len);
-	  z.zero();
-	  by_off[bit->first]->append( z );
-	} else {
-	  // we got none of this bx.  zero whole thing.
-	  assert(ox_off >= ox_len);
-	  ldout(cct, 21) << "  adding all zeros for this bit " << bit->second << dendl;
-	  bufferptr z(bit->second);
-	  z.zero();
-	  by_off[bit->first]->append( z );
-	}
-	ox_off += bit->second;
-      }
-      assert(ox_off == eit->length);
+      r.add_partial_result(cct, *bit, eit->buffer_extents);
     }
-    
-    // sort and string bits together
-    for (map<uint64_t, bufferlist*>::iterator it = by_off.begin();
-	 it != by_off.end();
-	 it++) {
-      assert(it->second->length());
-      if (it->first < (uint64_t)bytes_read) {
-	ldout(cct, 21) << "  concat buffer frag off " << it->first << " len " << it->second->length() << dendl;
-	bl->claim_append(*(it->second));
-      } else {
-	ldout(cct, 21) << "  NO concat zero buffer frag off " << it->first << " len " << it->second->length() << dendl;          
-      }
-      delete it->second;
-    }
-    
-    // trim trailing zeros?
-    if (bl->length() > bytes_read) {
-      ldout(cct, 10) << " trimming off trailing zeros . bytes_read=" << bytes_read 
-	       << " len=" << bl->length() << dendl;
-      bl->splice(bytes_read, bl->length() - bytes_read);
-      assert(bytes_read == bl->length());
-    }
-    
+    bl->clear();
+    r.assemble_result(cct, *bl, false);
   } else {
     ldout(cct, 15) << "  only one frag" << dendl;
-  
-    // only one fragment, easy
     bl->claim(resultbl[0]);
-    bytes_read = bl->length();
   }
-  
-  // finish, clean up
-  ldout(cct, 7) << " " << bytes_read << " bytes " 
-	  << bl->length()
-	  << dendl;
-    
+
   // done
+  uint64_t bytes_read = bl->length();
+  ldout(cct, 7) << "_sg_read_finish " << bytes_read << " bytes" << dendl;
+
   if (onfinish) {
     onfinish->finish(bytes_read);// > 0 ? bytes_read:m->get_result());
     delete onfinish;
@@ -2006,7 +2026,7 @@ void Objecter::ms_handle_remote_reset(Connection *con)
 void Objecter::dump_active()
 {
   ldout(cct, 20) << "dump_active .. " << num_homeless_ops << " homeless" << dendl;
-  for (hash_map<tid_t,Op*>::iterator p = ops.begin(); p != ops.end(); p++) {
+  for (map<tid_t,Op*>::iterator p = ops.begin(); p != ops.end(); p++) {
     Op *op = p->second;
     ldout(cct, 20) << op->tid << "\t" << op->pgid << "\tosd." << (op->session ? op->session->osd : -1)
 	    << "\t" << op->oid << "\t" << op->ops << dendl;
@@ -2029,7 +2049,7 @@ void Objecter::dump_requests(Formatter& fmt) const
 void Objecter::dump_ops(Formatter& fmt) const
 {
   fmt.open_array_section("ops");
-  for (hash_map<tid_t,Op*>::const_iterator p = ops.begin();
+  for (map<tid_t,Op*>::const_iterator p = ops.begin();
        p != ops.end();
        ++p) {
     Op *op = p->second;
